@@ -1,4 +1,6 @@
 from bs4 import BeautifulSoup
+from selenium import webdriver
+from selenium.webdriver.chrome.service import Service as ChromeService
 import lxml, os, copy, pandas, time, datetime, requests, sys
 
 class SSTime:
@@ -356,7 +358,7 @@ class SSSave:
             
             return filename
     
-    def __show_progress(self, current, total, endline = False):
+    def _show_progress(self, current, total, endline = False):
         
         if (self.progress_bar):
         
@@ -380,7 +382,7 @@ class SSSave:
             
             temp = 0
             
-            self.__show_progress(temp, length)
+            self._show_progress(temp, length)
             
             with open(self.filename, "wb") as writeFile:
                 
@@ -388,33 +390,25 @@ class SSSave:
                     
                     writeFile.write(data)
                     
-                    self.__show_progress(temp, length)
+                    self._show_progress(temp, length)
                     
                     temp += len(data)
                     
-            self.__show_progress(temp, length, endline = True)
+            self._show_progress(temp, length, endline = True)
         
         else:
             
             print("Warning [SSSave]: cannot download file because of unsupported arguments.\n")
         
-class StaticScraper:
-    
-    def help():
-        
-        print("StaticScraper(): ")
-        
-        print("\t[1]__init__(self, info)")
-        
-        print("\t[2]scrape(self, start = 1, pages = 1)\n")
+class ScraperBaseClass:
     
     def __init__(self, info):
         
         self.info = copy.deepcopy(info)
         
-        self.f_page = self.__format_webpage()
+        self.f_page = self._format_webpage()
     
-    def __format_webpage(self):
+    def _format_webpage(self):
         
         site, page = self.info.info["f_site"], self.info.info["f_page"]
         
@@ -432,7 +426,7 @@ class StaticScraper:
         
             return site + "/" + page
     
-    def __find_attributes(self, source, attr):
+    def _find_attributes(self, source, attr):
         
         data = source.find_all if (attr["multiple"]) else source.find
         
@@ -455,31 +449,11 @@ class StaticScraper:
             
         return data
     
-    def __scrape_webpage(self, page):
+    def _scrape_webpage(self, page):
         
-        webpage = self.f_page.format(page)
-        
-        data = BeautifulSoup(requests.get(webpage).text, "lxml")
-        
-        itemList = self.__find_attributes(data, self.info.info["f_item"])
-        
-        scraped = []
-        
-        for item in itemList:
-            
-            attrList = []
-            
-            for attr in self.info.info["f_attr"]:
-                
-                attrList.append(self.__find_attributes(item, attr))
-                
-            scraped.append(attrList)
-            
-        time.sleep(self.info.timesleep)
-        
-        return scraped
+        raise NotImplementedError("Must override ScraperBaseClass with StaticScraper or DynamicScraper!\n")
     
-    def __show_progress(self, page = None, progress = None, pages = None, finish = False):
+    def _show_progress(self, page = None, progress = None, pages = None, finish = False):
         
         
         if (finish):
@@ -500,9 +474,9 @@ class StaticScraper:
             
             progress = page - start + 1
             
-            self.__show_progress(page, progress, pages)
+            self._show_progress(page, progress, pages)
             
-            self.info.add_data(self.__scrape_webpage(page))
+            self.info.add_data(self._scrape_webpage(page))
             
             if (self.info.buffer_exceeded()):
                 
@@ -512,7 +486,7 @@ class StaticScraper:
                 
                 _start = False
             
-            self.__show_progress(finish = True)
+            self._show_progress(finish = True)
             
         print("")
             
@@ -521,6 +495,154 @@ class StaticScraper:
             self.info.save_data(_start)
             
             self.info.empty_data()
+    
+class StaticScraper(ScraperBaseClass):
+    
+    def help():
+        
+        print("StaticScraper(): ")
+        
+        print("\t[1] __init__(self, info)")
+        
+        print("\t[2] scrape(self, start = 1, pages = 1)\n")
+    
+    def __init__(self, info):
+        
+        super(StaticScraper, self).__init__(info)
+    
+    def _scrape_webpage(self, page):
+        
+        webpage = self.f_page.format(page)
+        
+        data = BeautifulSoup(requests.get(webpage).text, "lxml")
+        
+        itemList = self._find_attributes(data, self.info.info["f_item"])
+        
+        scraped = []
+        
+        for item in itemList:
+            
+            attrList = []
+            
+            for attr in self.info.info["f_attr"]:
+                
+                attrList.append(self._find_attributes(item, attr))
+                
+            scraped.append(attrList)
+            
+        time.sleep(self.info.timesleep)
+        
+        return scraped
+    
+class DynamicScraper(ScraperBaseClass):
+    
+    driver_names = {
+        "chrome" : webdriver.Chrome
+        #, 
+        #"firefox" : webdriver.Firefox   
+    }
+    
+    def help():
+        
+        print("DynamicScraper(): ")
+        
+        print("\t[1] __init__(self, info, driver = 'chrome', path = None, window = True)")
+        
+        print("\t[2] scrape(self, start = 1, pages = 1)\n")
+    
+    def __check_boolean(self, value):
+        
+        if (type(value) != bool):
+            
+            print(f"Warning [DynamicScraper]: {value} is not a boolean value.\n")
+            
+            if ((value == 0) or (value == "0") or (value == "f")):
+                
+                return False
+            
+        return value
+    
+    def __check_driver_path(self, path):
+        
+        if ((type(path) != None) and ((type(path) != str) or (not os.path.isfile(path)))):
+            
+            raise NameError(f"Webdriver {self.driver} could not be found at {path}.\n")
+            
+        return path
+    
+    def __check_driver_name(self, name):
+        
+        if ((type(name) == str) and (name.lower() in self.driver_names)):
+            
+            return name.lower()
+        
+        else:
+        
+            raise NameError(f"{name} is not a supported driver name.\n\tSupported: {list(self.driver_names.keys())}\n")
+    
+    def __init__(self, info, driver = "chrome", path = None, window = True):
+        
+        super(DynamicScraper, self).__init__(info)
+        
+        self.driver = self.__check_driver_name(driver)
+    
+        self.path = self.__check_driver_path(path)
+        
+        self.window = self.__check_boolean(window)
+        
+    def __get_driver(self, name):
+        
+        return self.driver_names[name]
+    
+    def __get_parameter(self):
+        
+        op = {}
+        
+        if (self.path != None):
+            
+            _op = ChromeService(self.path)
+            
+            op["service"] = _op
+            
+        if (self.window == False):
+            
+            _op = webdriver.ChromeOptions()
+            
+            _op.add_argument("headless")
+            
+            op["options"] = _op
+                
+        return op
+    
+    def _scrape_webpage(self, page):
+        
+        webpage = self.f_page.format(page)
+        
+        driver = self.__get_driver(self.driver)(
+            **(self.__get_parameter())
+        )
+        
+        driver.get(webpage)
+        
+        data = BeautifulSoup(driver.page_source, "lxml")
+        
+        itemList = self._find_attributes(data, self.info.info["f_item"])
+        
+        scraped = []
+        
+        for item in itemList:
+            
+            attrList = []
+            
+            for attr in self.info.info["f_attr"]:
+                
+                attrList.append(self._find_attributes(item, attr))
+                
+            scraped.append(attrList)
+            
+        time.sleep(self.info.timesleep)
+        
+        return scraped
 """
 def format_price(price):
     return eval(''.join([ x for x in price if x.isnumeric() or x == '.']))
@@ -532,7 +654,7 @@ itemformat = SSFormat(
     element_type = "li", search_type = "class_", search_clue = "col-xs-6 col-sm-4 col-md-3 col-lg-3", multiple = True
 )
 attrList = [
-    SSFormat(element_type = "p", search_type = "class_", search_clue = "price_color", extract = "text", nickname = "price", format = format_price, filter = filter_price, keep = False),
+    SSFormat(element_type = "p", search_type = "class_", search_clue = "price_color", extract = "text", nickname = "price", format = format_price, filter = filter_price),
     SSFormat(element_type = "a", extract = "href", nickname = "url")
 ]
 info = SSInfo(website, pageformat, itemformat, attrList)
@@ -541,6 +663,21 @@ scraper = StaticScraper(data)
 scraper.scrape(start = 1, pages = 20)
 """
 """
+DynamicScraper.help()
+website = "https://www.ebay.com/sch/i.html?_from=R40&_nkw=processor&_sacat=0"
+pageformat = "&_pgn={}"
+itemformat = SSFormat(
+    element_type = "li", search_type = "class_", search_clue = "s-item s-item__pl-on-bottom s-item--watch-at-corner", multiple = True
+)
+attrList = [
+    SSFormat(element_type = "a", search_type = "class_", search_clue = "s-item__link", extract = "href")        
+]
+info = SSInfo(website, pageformat, itemformat, attrList)
+data = SSData(info, timesleep = 0)
+scraper = DynamicScraper(data, driver = "Chrome", path = "../../../chromedriver.exe", window = False)
+scraper.scrape(start = 1, pages = 4)
+"""
+"""
 SSSave.help()
 
 file = SSSave("temp.json", "link", progress_bar = True, overwrite = False)
